function [dS] = DiffEqWrapper(t,S,magnet,Mass_constant,Mass_var, C_var, G_var, M,I,L,R,g)  %S=state vector
%[dS] = DiffEqWrapper(t,S,magnet,Mass_constant,Mass_var, C_var, G_var)
%    Solve differntial equations of motion for 2-link Monkey Bot during a
%    single time step
%
%   S=state vector: [x y t1 t2 dx dy dt1 dy2]'
%   dS=time derivate of state vector (including accelerations):
%               ds=[dx dy dt1 dt2 ddx ddy ddt1 ddt2]'
%
%  t-> time elapsed
%  magnet -> current magnet state
%   Mass_constant, Mass_var, C_var, G_var -> precalulated poritions of
%   M,C,g matricies
%
%  David Ufford
%  June 2009



global motorState;  %pull the state the motor is in


%%% Friction Parameters

magnetFriction_v=0;%viscous values of friction are proportional to angular velocity
jointFriction_v=0;
magnetFriction_c=0; %constant values of friction to be applied
jointFriction_c=0;

%%%%Pull out desired vars from state vector S
%q=S(1:4); %x,y,theta1, theta2
dq=S(5:end); %dx, dy, dtheta1, dtheta2
%x=1, y=2
t1=S(3); %theta 1
t2=S(4); %theta 2
%dx=5, dy=6
dt1=S(7); %d_theta1
dt2=S(8); %d_theta2
         
    

%Need to generate Mass, C, G, and F matrices
%motorState=5*sin(1/sqrt(5/9.8)*t);
if(magnet>0 && magnet<3)
    %since we have a single fixed magnet point, we can use the 
    %reduced matrices (only 2 equations0
    %to save on some computation time.
    
    %%%%%%%%%calculate M,C,G matrices from state and pre_calced portions
    Mass=Mass_constant+Mass_var.*cos(t2); 
    C=(C_var.*sin(t2).*[dt2 dt1+dt2; dt1 0])*S(7:8);
    G=G_var*[sin(t1); sin(t1+t2)];
    
    %%%%%External Force vector = friction at joints + motor torque
    F=[-magnetFriction_v*dt1 + -magnetFriction_c*sign(dt1);
        -jointFriction_v*dt2+motorState - jointFriction_c*sign(dt2)];

    %solve equations of motion
    ddq=linsolve(Mass,F-C-G);
    ddq=[0; 0; ddq];%feed zeros in for x/y acceleration
elseif(magnet==3)
    %both magnets fixed (untested)....
    ddq=[0; 0; 0; 0];
else

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
    %% - FREE FLIGHT - use all equations
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    C1=cos(t1); %precalc some sins/cosines
    C12=cos(t1+t2);
    C2=cos(t2);
    S1=sin(t1);
    S12=sin(t1+t2);
    S2=sin(t2);
    mrml=M(1)*R(1)+M(2)*L(1);
    Mass=[M(1)+M(2)   0   (mrml)*C1+M(2)*R(2)*C12  M(2)*R(2)*C12;
        0       M(1)+M(2)   (mrml)*S1+M(2)*R(2)*S12  M(2)*R(2)*S12;
        (mrml)*C1+M(2)*R(2)*C12          (mrml)*S1+M(2)*R(2)*S12 ...
        I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+M(2)*R(2)^2+2*M(2)*R(2)*L(1)*C2         M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2);
        M(2)*R(2)*C12       M(2)*R(2)*S12       M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2)      I(2)+M(2)*R(2)^2];
    C=[0 0 -((mrml)*S1*dt1+(M(2)*R(2)*S12)*(dt1+dt2))         -M(2)*R(2)*S12*(dt1+dt2);
        0 0 (mrml)*C1*dt1+(M(2)*R(2)*C12)*(dt1+dt2)            M(2)*R(2)*C12*(dt1+dt2);
        0 0  -M(2)*R(2)*L(1)*S2*dt2                                     -M(2)*R(2)*L(1)*S2*(dt1+dt2);
        0 0  M(2)*R(2)*L(1)*S2*dt1                                            0];
    Cq=C*dq;
    G= [ 0;
        g*(M(1)+M(2));
        g*(mrml)*S1+g*M(2)*R(2)*S12;
        g*M(2)*R(2)*S12];
    F=[0; 0; 0; motorState];

    %free flight - use all set of 4 eqs
    ddq=linsolve(Mass,F-Cq-G);
    %Pddq=P*invM*(u-Cdq-g)

    %The equation is F=M(q)*ddq + C(q,dq)*dq + G(q);
    % solves to M*ddq=F-C*dq-Gq

end

dS=[dq; ddq];%return compiled dS vector


