function [final_t, final_Q]=monkeySim(Q0,timeLimit, M, I, L, R, g)
%[t, Q]=monkeySim(Q0,timeLimit, M, I, L, R, g)
% This is the primary function to run the monkey-bot simulation.  
% A 2-link swinging robot is modeled with given parameters and intial
% conditions.  
%
%  t, Q -> time/Q arrays for points during simulation
%
%   Q0 -> initial conditions
%    timelimit -> max length of simulation
%    MILRg -> system paramters
%
%  David Ufford
%  June 2009

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Procedure overview:
1.  Setup vars and function handles to differtial equation function
2.  Precalculate a few matricies for improved performance
3.  Call ode45 to numerically integrate equations of motions in current state
       3.1  ode calls DiffEqWrapper to determine values of EoM and numerically
             integrates to determine motion of monkeybot
       3.2  For each step, ode also calls the "output" function DiffEqMonitor
            DiffEqMonitor analyzes each .001 second of time and
                -draws the system (if desired)
                -calls control algorithm
       3.3  Control algorithm can terminate the integration of ode45.  
            This should occur when the magnets are swapped or motion type changes,
            as post-impact velocties need to be calculated and there will be 
            discontinuties that ode45 may hiccup on
4 When ode45 terminates, check if a magnet/phase transition happened
5 If the fixed magnet changed (magnet swap)
    5.1  Flip the system so that the bottom link becomes the top link 
            (and vise versa), 
    5.2 Apply impact projection matrix to determine new velocities
6 Reset initial conditions and call ode45 again, until timelimit is reached
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}


clear global next_swap;%ensure these are cleared between runs
clear global magnetState;
clear global motorState;

%%%% intialize variables
global magnetState;%0 = magnets off, %1 = magnet 1 on.  %2= magnet 2 on %3=both magnets on
global motorState;%= current value of torque being applied
global terminated_t;%method of passing the current state when control algorimthm
global terminated_Q;%  determines a ode45 termination is necessary
global doplot;

DQcheck=@(t,Q,flag)DiffEqMonitor(t,Q,flag,M,I,L,R); %monitoring function

%ode options.  Error tolerance has been tightened
options = odeset('RelTol',1e-9,'AbsTol',1e-9,'OutputFcn',DQcheck);

%doublecheck doplot is set
if(~exist('doplot') || length(doplot)<1)
    doplot=0;
end

t=0;  %current time
fixedEnd=1;  %start with Magnet '1' is on top
magnetState=1;  %and it is fixed to the wall
motorState=0; %start with motor off
terminated_t=[];
terminated_Q=[];

%precalc some stuff
m2r2=M(2)*R(2)^2;
Mass_constant=[I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+m2r2,        m2r2+I(2);
    m2r2+I(2)       I(2)+m2r2];%constants added on to mass matrix
Mass_var=[2*M(2)*R(2)*L(1), M(2)*R(2)*L(1);
    M(2)*R(2)*L(1) 0];%constants multiplied by q for mass matrix
C_var=[-M(2)*R(2)*L(1)       -M(2)*R(2)*L(1);
    M(2)*R(2)*L(1)     0];
G_var=[g*(M(1)*R(1)+M(2)*L(1)) g*M(2)*R(2);
    0 g*M(2)*R(2)];

Q=Q0;
final_t=[];
final_Q=[];

disp('Solving....');

while (t<timeLimit-.1)%loop while we are still within time limit
    
    %create function handle to DiffEqWrapper with correct parameters
    DiffEq=@(t,Q)DiffEqWrapper(t,Q,magnetState,Mass_constant,Mass_var,C_var,G_var, M,I,L,R,g);  %reduce number of parameters

    tspan=t:.001:timeLimit; %setup time span -> request intervals of .001
                               %(these intervals only impact reported
                               %     values, not accuracy of integration)
                               
    [T, state]=ode45(DiffEq,tspan, Q,options); %--->GO!
    t=T(end);
    Q=state(end,:);


    %%%%CHECK FOR IMPACTS
    if(magnetState >0 && magnetState~=fixedEnd)
        %magnet needs to be swapped

        %integration was terminated, so grab the values at
        %terminated time (stored in globals)
        t=terminated_t;
        Q=terminated_Q;
        
        %NET1=calcEnergy(Q,M,I,L,R,g); calculate pre-impact energys

        %%%%5.1 Reposition variables so that other end is the base x/y
        %         (flip system)
        [Q M I L R]=flipEnds(Q,M,I,L,R);

        %%%%5.2  Calculate post-impact velocties (impact at the new x/y)
        dq=Q(5:8);
        P=buildP(Q,M,I,L,R);%Build Projection Matrix
        dq=P*dq;  %...calculate post-impact velocities (post)=P*(pre)
        Q=[Q(1:4); dq]; %compile new state vector
        
        %NET2=calcEnergy(Q,M,I,L,R,g); post-impact energy
        %disp(['transition at' num2str(t)]);

        fixedEnd=magnetState; %swap the end that is attached to the magent
        
        
        %make sure we toss out all of the calculated values from time after
        %the swap
        ii = find(T==terminated_t);
        T(ii:end)=[];
        state(:,ii:end)=[];
    end
    final_t=[final_t; T];
    final_Q=[final_Q; state];

end %loop until we reached the timeLimit for simulation
disp('Done!');


function P=buildP(Q,M,I,L,R)
%Builds the projection matrix used during impact

t1=Q(3);
t2=Q(4);

%sins/cosines
C1=cos(t1);
C12=cos(t1+t2);
C2=cos(t2);
S1=sin(t1);
S12=sin(t1+t2);
S2=sin(t2);

mrml=M(1)*R(1)+M(2)*L(1);

Mass=[M(1)+M(2)   0   (mrml)*C1+M(2)*R(2)*C12  M(2)*R(2)*C12;
    0       M(1)+M(2)   (mrml)*S1+M(2)*R(2)*S12  M(2)*R(2)*S12;
    (mrml)*C1+M(2)*R(2)*C12          (mrml)*S1+M(2)*R(2)*S12 ...
    I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+M(2)*R(2)^2+2*M(2)*R(2)*L(1)*C2         M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2);
    M(2)*R(2)*C12       M(2)*R(2)*S12       M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2)      I(2)+M(2)*R(2)^2];

invM=inv(Mass);

J=[1 0 0 0;
    0 1 0 0];

P=eye(4)-invM*(J.')*inv(J*invM*(J.'))*J;
